@@ -466,7 +466,12 @@ class APIConnection:
         msg_types = [HelloResponse]
         if login:
             messages.append(self._make_connect_request())
-            msg_types.append(ConnectResponse)
+            if self._params.password:
+                # Only wait for ConnectResponse if we actually have
+                # a password to send, but we will still register
+                # a handler for a ConnectResponse just in case
+                # the device has a password but we don't expect it
+                msg_types.append(ConnectResponse)
 
         responses = await self.send_messages_await_response_complex(
             tuple(messages),
@@ -476,9 +481,10 @@ class APIConnection:
             tuple(msg_types),
             CONNECT_REQUEST_TIMEOUT,
         )
+
         resp = responses.pop(0)
         self._process_hello_resp(resp)
-        if login:
+        if login and self._params.password:
             login_response = responses.pop(0)
             self._process_login_response(login_response)
 
@@ -487,6 +493,11 @@ class APIConnection:
         if login_response.invalid_password:
             raise InvalidAuthAPIError("Invalid password!")
 
+    def _handle_login_response(self, login_response: ConnectResponse) -> None:
+        """Handle a ConnectResponse."""
+        if login_response.invalid_password:
+            self.report_fatal_error(InvalidAuthAPIError("Invalid password!"))
+
     def _process_hello_resp(self, resp: HelloResponse) -> None:
         """Process a HelloResponse."""
         if self._debug_enabled:
@@ -602,8 +613,9 @@ class APIConnection:
         except (Exception, CancelledError) as ex:
             # If the task was cancelled, we need to clean up the connection
             # and raise the CancelledError as APIConnectionError
-            self._cleanup()
-            raise self._wrap_fatal_connection_exception("resolving", ex)
+            fatal_exc = self._wrap_fatal_connection_exception("resolving", ex)
+            self.report_fatal_error(fatal_exc)
+            raise fatal_exc
         finally:
             self._set_resolve_host_future()
         self._set_connection_state(CONNECTION_STATE_HOST_RESOLVED)
@@ -636,8 +648,9 @@ class APIConnection:
         except (Exception, CancelledError) as ex:
             # If the task was cancelled, we need to clean up the connection
             # and raise the CancelledError as APIConnectionError
-            self._cleanup()
-            raise self._wrap_fatal_connection_exception("starting", ex)
+            fatal_exc = self._wrap_fatal_connection_exception("starting", ex)
+            self.report_fatal_error(fatal_exc)
+            raise fatal_exc
         finally:
             self._set_start_connect_future()
         self._set_connection_state(CONNECTION_STATE_SOCKET_OPENED)
@@ -706,8 +719,9 @@ class APIConnection:
         except (Exception, CancelledError) as ex:
             # If the task was cancelled, we need to clean up the connection
             # and raise the CancelledError as APIConnectionError
-            self._cleanup()
-            raise self._wrap_fatal_connection_exception("finishing", ex)
+            fatal_exc = self._wrap_fatal_connection_exception("finishing", ex)
+            self.report_fatal_error(fatal_exc)
+            raise fatal_exc
         finally:
             self._set_finish_connect_future()
         self._set_connection_state(CONNECTION_STATE_CONNECTED)
@@ -897,11 +911,20 @@ class APIConnection:
         if self._fatal_exception is None:
             if self._expected_disconnect is False:
                 # Only log the first error
-                _LOGGER.warning(
+                log_level = (
+                    logging.DEBUG
+                    if isinstance(err, APIConnectionCancelledError)
+                    else logging.WARNING
+                )
+                _LOGGER.log(
+                    log_level,
                     "%s: Connection error occurred: %s",
                     self.log_name,
                     err or type(err),
-                    exc_info=not str(err),  # Log the full stack on empty error string
+                    exc_info=not str(err)
+                    or isinstance(
+                        err, UnhandledAPIConnectionError
+                    ),  # Log the full stack on empty error string
                 )
 
             # Only set the first error since otherwise the original
@@ -1009,6 +1032,9 @@ class APIConnection:
         self._add_message_callback_without_remove(
             self._handle_get_time_request_internal, (GetTimeRequest,)
         )
+        self._add_message_callback_without_remove(
+            self._handle_login_response, (ConnectResponse,)
+        )
 
     def _handle_disconnect_request_internal(  # pylint: disable=unused-argument
         self, _msg: DisconnectRequest
