@@ -58,6 +58,59 @@ from .common import (
 KEEP_ALIVE_TIMEOUT_RATIO = 4.5
 
 
+async def test_connect_hello_login_with_login_false_and_password_none(
+    conn: APIConnection,
+    resolve_host,
+    aiohappyeyeballs_start_connection,
+) -> None:
+    """Test that _connect_hello_login works with login=False and password=None.
+
+    This test verifies the fix for the issue where _connect_hello_login would
+    try to pop from an empty list when login=False but password=None.
+    """
+    loop = asyncio.get_running_loop()
+    transport = MagicMock()
+    connected = asyncio.Event()
+    protocol: APIPlaintextFrameHelper | None = None
+
+    async def _create_connection(*args, **kwargs):
+        nonlocal protocol
+        transport_proto = _create_mock_transport_protocol(
+            transport,
+            connected,
+            lambda: APIPlaintextFrameHelper(
+                connection=conn,
+                client_info="test",
+                log_name=conn.log_name,
+            ),
+        )
+        protocol = transport_proto[1]
+        return transport, protocol
+
+    with patch.object(loop, "create_connection", side_effect=_create_connection):
+        # Use the connect helper which properly handles the connection flow
+        connect_task = asyncio.create_task(connect(conn, login=False))
+
+        # Wait for connection to be established
+        await connected.wait()
+
+        # Send only HelloResponse (no ConnectResponse since login=False)
+        assert protocol is not None
+        hello_response = bytes.fromhex(
+            "003602080110091a216d6173746572617672656c61792028657"
+            "370686f6d652076323032332e362e3329220d6d617374657261"
+            "7672656c6179"
+        )
+        mock_data_received(protocol, hello_response)
+
+        # The connection should complete successfully without trying to pop
+        # a ConnectResponse from an empty list
+        await connect_task
+
+        assert conn.is_connected
+        assert conn.api_version is not None
+
+
 async def test_connect(
     plaintext_connect_task_no_login: tuple[
         APIConnection, asyncio.Transport, APIPlaintextFrameHelper, asyncio.Task
@@ -591,6 +644,48 @@ async def test_connect_correct_password(
     assert conn.is_connected
 
 
+async def test_connect_password_required_but_not_sent(
+    conn: APIConnection,
+    resolve_host,
+    aiohappyeyeballs_start_connection,
+) -> None:
+    """Test that when the device requires a password but we don't send one, we get an auth error."""
+    transport = MagicMock()
+    connected = asyncio.Event()
+    event_loop = asyncio.get_running_loop()
+
+    with patch.object(
+        event_loop,
+        "create_connection",
+        side_effect=partial(_create_mock_transport_protocol, transport, connected),
+    ):
+        # Note: login=False because conn fixture has no password
+        connect_task = asyncio.create_task(connect(conn, login=False))
+        await connected.wait()
+
+        protocol = conn._frame_helper
+
+        # Send hello response
+        send_plaintext_hello(protocol)
+
+        # Complete the connection first since we're not expecting a ConnectResponse
+        await connect_task
+        assert conn.is_connected
+
+        # Device sends ConnectResponse with invalid_password=True
+        # This simulates a device that requires a password but we didn't send one
+        send_plaintext_connect_response(protocol, True)
+
+        # Give the event loop a chance to process the message
+        await asyncio.sleep(0)
+
+        # The connection should now be disconnected due to the auth error
+        assert not conn.is_connected
+        # Check that the fatal exception was set
+        assert isinstance(conn._fatal_exception, InvalidAuthAPIError)
+        conn.force_disconnect()
+
+
 async def test_connect_wrong_version(
     plaintext_connect_task_with_login: tuple[
         APIConnection, asyncio.Transport, APIPlaintextFrameHelper, asyncio.Task
@@ -1126,3 +1221,53 @@ async def test_internal_message_received_immediately_after_connection(
 
         # Clean up
         conn.force_disconnect()
+
+
+@pytest.mark.asyncio
+async def test_report_fatal_error_cancellation_logged_at_debug_level(
+    conn: APIConnection,
+    caplog: pytest.LogCaptureFixture,
+) -> None:
+    """Test that APIConnectionCancelledError is logged at DEBUG level in report_fatal_error."""
+    with caplog.at_level(logging.DEBUG):
+        # Report a cancelled error
+        cancelled_error = APIConnectionCancelledError("Connection was cancelled")
+        conn.report_fatal_error(cancelled_error)
+
+    # Check that the error was logged at DEBUG level
+    assert "Connection error occurred" in caplog.text
+    assert "Connection was cancelled" in caplog.text
+
+    # Verify it was logged at DEBUG level
+    matching_records = [
+        record
+        for record in caplog.records
+        if "Connection error occurred" in record.message
+    ]
+    assert len(matching_records) == 1
+    assert matching_records[0].levelno == logging.DEBUG
+
+
+@pytest.mark.asyncio
+async def test_report_fatal_error_regular_errors_logged_at_warning_level(
+    conn: APIConnection,
+    caplog: pytest.LogCaptureFixture,
+) -> None:
+    """Test that regular errors are still logged at WARNING level in report_fatal_error."""
+    with caplog.at_level(logging.DEBUG):
+        # Report a regular connection error
+        regular_error = APIConnectionError("Regular connection error")
+        conn.report_fatal_error(regular_error)
+
+    # Check that the error was logged at WARNING level
+    assert "Connection error occurred" in caplog.text
+    assert "Regular connection error" in caplog.text
+
+    # Verify it was logged at WARNING level
+    matching_records = [
+        record
+        for record in caplog.records
+        if "Connection error occurred" in record.message
+    ]
+    assert len(matching_records) == 1
+    assert matching_records[0].levelno == logging.WARNING
